@@ -1,18 +1,20 @@
-use crate::item_sort_list::CommitMethod;
-use crate::main_window::{CommitMethodValues, ImageSieve};
-use num_traits::{FromPrimitive, ToPrimitive};
+use crate::item_sort_list::{DirectoryNames, SieveMethod};
+use crate::main_window::{ImageSieve, SieveComboValues};
 use serde::{Deserialize, Serialize};
-use sixtyfps::{ComponentHandle, Model, ModelHandle, SharedString};
+use sixtyfps::{ComponentHandle, ModelHandle, SharedString};
+
+use super::model_to_enum::{enum_to_model, model_to_enum};
 
 #[derive(Clone, Serialize, Deserialize)]
 pub struct Settings {
     pub source_directory: String,
     pub target_directory: String,
-    pub commit_method: CommitMethod,
+    pub sieve_method: SieveMethod,
     pub use_timestamps: bool,
     pub timestamp_max_diff: i64,
     pub use_hash: bool,
     pub hash_max_diff: u32,
+    pub sieve_directory_names: Option<DirectoryNames>,
 }
 
 impl Settings {
@@ -20,21 +22,24 @@ impl Settings {
         Self {
             source_directory: String::new(),
             target_directory: String::new(),
-            commit_method: CommitMethod::Copy,
+            sieve_method: SieveMethod::Copy,
             use_timestamps: true,
             timestamp_max_diff: 5,
             use_hash: false,
             hash_max_diff: 8,
+            sieve_directory_names: Some(DirectoryNames::YearAndMonth),
         }
     }
 
     pub fn from_window(window: &ImageSieve) -> Self {
         //TODO: Also save last selected image and restart there
+        let methods: ModelHandle<SharedString> = window.global::<SieveComboValues>().get_methods();
+        let directory_names: ModelHandle<SharedString> =
+            window.global::<SieveComboValues>().get_directory_names();
         Settings {
             source_directory: window.get_source_directory().to_string(),
             target_directory: window.get_target_directory().to_string(),
-            commit_method: FromPrimitive::from_i32(window.get_commit_method())
-                .unwrap_or(CommitMethod::Copy),
+            sieve_method: model_to_enum(&methods, &window.get_sieve_method()),
             use_timestamps: window.get_use_timestamps(),
             timestamp_max_diff: convert_timestamp_difference(
                 &window.get_timestamp_difference().to_string(),
@@ -44,22 +49,31 @@ impl Settings {
             hash_max_diff: convert_sensitivity_to_u32(
                 &window.get_similarity_sensitivity().to_string(),
             ),
+            sieve_directory_names: Some(model_to_enum(
+                &directory_names,
+                &window.get_sieve_directory_names(),
+            )),
         }
     }
 
     pub fn to_window(&self, window: &ImageSieve) {
         window.set_source_directory(SharedString::from(self.source_directory.clone()));
         window.set_target_directory(SharedString::from(self.target_directory.clone()));
-        let commit_index = ToPrimitive::to_i32(&self.commit_method).unwrap();
-        window.set_commit_method(commit_index);
-        let values: ModelHandle<SharedString> = window.global::<CommitMethodValues>().get_values();
-        window.set_commit_method_value(values.row_data(commit_index as usize));
+        let methods: ModelHandle<SharedString> = window.global::<SieveComboValues>().get_methods();
+        window.set_sieve_method(enum_to_model(&methods, &self.sieve_method));
         window.set_use_timestamps(self.use_timestamps);
         window.set_timestamp_difference(SharedString::from(self.timestamp_max_diff.to_string()));
         window.set_use_similarity(self.use_hash);
         window.set_similarity_sensitivity(SharedString::from(convert_u32_to_sensitivity(
             self.hash_max_diff,
         )));
+        let directory_names: ModelHandle<SharedString> =
+            window.global::<SieveComboValues>().get_directory_names();
+        let directory_name = self
+            .sieve_directory_names
+            .as_ref()
+            .unwrap_or(&DirectoryNames::YearAndMonth);
+        window.set_sieve_directory_names(enum_to_model(&directory_names, directory_name));
     }
 }
 
